@@ -24,13 +24,14 @@ def _generate_storage_path(powerlink_key: str, task_key: str, app_name: str) ->
         return storage_path
 
 #===================================================================================================
+#TODO AGENT_NAME can be empty if actions will use the cookie
 class Core:
     """Handles the calls to core APIs for task authentication, execution, and lifecycle management."""
 
     def __init__(
         self, 
-        AGENT_NAME: str, 
-        CORE_API: str, 
+        AGENT_NAME: str | None = None, 
+        CORE_API: str = None, 
         JWT_TOKEN: str | None = None,
     ):
         """
@@ -46,8 +47,8 @@ class Core:
             ValueError: If AGENT_NAME or CORE_API is missing or empty.
         """
 
-        if not AGENT_NAME:
-            raise ValueError("[INIT] AGENT_NAME is required and cannot be empty.")
+        # if not AGENT_NAME:
+        #     raise ValueError("[INIT] AGENT_NAME is required and cannot be empty.")
         if not CORE_API:
             raise ValueError("[INIT] CORE_API is required and cannot be empty.")
 
@@ -55,6 +56,7 @@ class Core:
         self.AGENT_NAME = AGENT_NAME
         self.JWT_TOKEN = JWT_TOKEN
         self._PASSWORD: str = None  # Store password for future use
+        self.USER_INFO: dict = {}
         self.RUNTIME_VARIABLES: dict[str, str | None] = {}
     
     def is_token_valid(self) -> bool:
@@ -227,6 +229,7 @@ class Core:
                         if login_response.status == 200 and "jwt" in login_json:
                             self.JWT_TOKEN = login_json["jwt"]
                             self._PASSWORD = PASSWORD  # Store password for future use
+                            self.USER_INFO = login_json.get("user", {})
                             os.environ["AGENTIX_CORE_JWT_TOKEN"] = self.JWT_TOKEN
                             logger.info(f"✅ [LOGIN] Login succeeded on attempt {attempt}")                            
                             break
@@ -350,9 +353,14 @@ class Core:
                         logger.info(f"<<<<< ✅ [START-TASK] Task Started!")
                         return response_json
                     else:
-                        logger.error(f"❌ [START-TASK] Failed to start task. HTTP Status: {response.status} - {response_json}")
-                        raise RuntimeError(f"❌ [START-TASK] Failed to start task. HTTP Status: {response.status} - {response_json}")
-
+                        redirect_url = response.headers.get("Location")
+                        redirect_note = f"Redirect to: {redirect_url}" if redirect_url else "No redirect location provided."
+                        
+                        logger.error(f"❌ [START-TASK] Failed to start task. "
+                                    f"HTTP Status: {response.status} - {response_json} - {redirect_note}")
+                        
+                        raise RuntimeError(f"❌ [START-TASK] Failed to start task. HTTP Status: {response.status} - {response_json} - {redirect_note}")
+                    
         except Exception as e:
             logger.error(f"❌ [START-TASK] Error calling Start Task API: {e}")
             raise RuntimeError(f"❌ [START-TASK] Error calling Start Task API: {e}")
@@ -360,7 +368,7 @@ class Core:
     #================================================================================================
     # Submit Task in Core
     #================================================================================================
-    async def submit_task(self, JOB_COOKIE: dict, task_key: str, output: list[dict] | None = None, metadata: str = None) -> bool:
+    async def submit_task(self, JOB_COOKIE: dict, task_key: str, output: list[dict] | None = None, metadata: str = None, notifiy_launchpad_worker: bool = False) -> bool:
         """
         Submits a task in Core by calling the `/submit` endpoint.
 
@@ -387,7 +395,7 @@ class Core:
         try:
             logger.info(f">>> [SUBMIT-TASK] Submit Task: {task_key}")
 
-            url = f"{self.CORE_API}/v1/tasks/{task_key}/submit?noRedirect=true"
+            url = f"{self.CORE_API}/v1/tasks/{task_key}/submit?noRedirect=true&notifiyLaunchpadWorker={notifiy_launchpad_worker}"
             headers = {"Referer": f"{self.CORE_API}"}
             request_body = {
                 "data": {
@@ -423,7 +431,7 @@ class Core:
     #================================================================================================
     # Reject Task in Core
     #================================================================================================
-    async def reject_task(self, JOB_COOKIE: dict, task_key: str, rejection_reason: str = "No reason provided", metadata: str = None) -> bool:
+    async def reject_task(self, JOB_COOKIE: dict, task_key: str, rejection_reason: str = "No reason provided", metadata: str = None, notifiy_launchpad_worker: bool = False) -> bool:
         """
         Rejects a task in Core by calling the `/reject` endpoint.
 
@@ -451,7 +459,7 @@ class Core:
         try:
             logger.info(f">>> Reject Task: {task_key}")
 
-            url = f"{self.CORE_API}/v1/tasks/{task_key}/reject?noRedirect=true"
+            url = f"{self.CORE_API}/v1/tasks/{task_key}/reject?noRedirect=true&notifiyLaunchpadWorker={notifiy_launchpad_worker}"
             headers = {"Referer": f"{self.CORE_API}"}
             request_body = {
                 "data": {
@@ -670,6 +678,7 @@ class Core:
             logger.error(f"❌ [NEW-JOB] Exception during job creation: {e}")
             raise RuntimeError(f"❌ [NEW-JOB] Exception during job creation: {e}")
 
+
     #================================================================================================
     # Start Job in Core
     #================================================================================================
@@ -727,6 +736,59 @@ class Core:
             raise RuntimeError(f"❌ [START-JOB] Error: {e}")
 
 
+
+    #================================================================================================
+    # Notify Job in Core
+    #================================================================================================
+    async def notify_job (self, job_key: str, metadata: str = "") -> dict:
+        """
+        Notifies the Core system about the current job status.
+
+        Args:
+            job_key (str): The job identifier.
+            metadata (str): Optional metadata to send along with the notification.
+
+        Returns:
+            dict: A dictionary containing the API response.
+
+        Raises:
+            RuntimeError: If the notification fails or an exception occurs.
+        """
+        logger.info("🔔 [NOTIFY-JOB] Sending job notification to Core...")
+
+        if not self.JWT_TOKEN:
+            raise ValueError("❌ [NOTIFY-JOB] Must login to Core before notifying a job.")
+
+        url = f"{self.CORE_API}/v1/jobs/{job_key}/notify"  # 🛠️ Fix: missing slash before job_key
+        headers = {
+            "Authorization": f"Bearer {self.JWT_TOKEN}",
+            "Referer": self.CORE_API,
+            "Content-Type": "application/json"
+        }
+
+        payload = {
+            "data": {
+                "metadata": metadata
+            }
+        }
+
+        try:
+            async with aiohttp.ClientSession() as session:
+                async with session.post(url, headers=headers, json=payload) as response:
+                    content_type = response.headers.get("Content-Type", "")
+                    response_json = await response.json() if "application/json" in content_type else {}
+
+                    if response.status == 200:
+                        logger.info("✅ [NOTIFY-JOB] Job notification sent successfully.")
+                        return response_json
+                    else:
+                        logger.error(f"❌ [NOTIFY-JOB] Failed. Status: {response.status}, Response: {response_json}")
+                        raise RuntimeError(f"[NOTIFY-JOB] Failed with status {response.status}. Response: {response_json}")
+        except Exception as e:
+            logger.exception(f"❌ [NOTIFY-JOB] Exception occurred: {e}")
+            raise RuntimeError(f"[NOTIFY-JOB] Exception: {e}")
+        
+
     #================================================================================================
     # Save File to task
     #================================================================================================
@@ -853,7 +915,8 @@ class Core:
         input_data: dict,
         output_data: dict,
         status: str = "SUCCESS",
-        description: str = ""
+        description: str = "",
+        type: str = ""
     ) -> bool:
         """
         Adds a task action entry for a specific task in Core.
@@ -870,6 +933,7 @@ class Core:
             output_data (dict): Dictionary to send as the action output.
             status (str): Status of the action, default is "SUCCESS".
             description (str): Optional description of the action.
+            type (str): Optional type of the action.
 
         Returns:
             bool: True if action is added successfully.
@@ -898,7 +962,8 @@ class Core:
                     "destinationName": destination_name,
                     "input": input_data,
                     "status": status,
-                    "output": output_data
+                    "output": output_data,
+                    "type": type
                 }
             }
 
